/*******************************************************************************
      Voltmetro_4C_Lcd
      
      c/ ATtiny2313

      Autor: Cludio Larios
       
      Data: 18/08/2016
      Metodo: Delta_Sigma (12 bits)
      C/ 4 canais multiplexados com integrado 4066
      Configure para osc_interno de 8 Mhz
      Uso didtico
      Use MikroC for Avr compiler 6.1.0
      Este arquivo  parte integrante do Blog Lrios.Tecnologia.ws

*******************************************************************************/
 // Conexes do mdulo Lcd
sbit LCDRS at PORTB3_bit;
sbit LCDEN at PORTB2_bit;
sbit LCDD4 at PORTB4_bit;
sbit LCDD5 at PORTB5_bit;
sbit LCDD6 at PORTB6_bit;
sbit LCDD7 at PORTB7_bit;
//Direo dos pinos do Lcd
sbit LCDRS_Dir at DDB3_bit;
sbit LCDEN_Dir at DDB2_bit;
sbit LCDD4_Dir at DDB4_bit;
sbit LCDD5_Dir at DDB5_bit;
sbit LCDD6_Dir at DDB6_bit;
sbit LCDD7_Dir at DDB7_bit;
// End LCD module connections

sbit AIN1_DIR at DDB1_bit;  //entrada do comparador ain1
sbit AIN0_DIR at DDB0_bit;  //entrada do comparador ain0
sbit CANAL1 at PORTD4_bit;
sbit CANAL2 at PORTD1_bit;
sbit CANAL3 at PORTA1_bit;
sbit CANAL4 at PORTA0_bit;
sbit CANAL1_Dir at DDD4_bit;
sbit CANAL2_Dir at DDD1_bit;
sbit CANAL3_Dir at DDA1_bit;
sbit CANAL4_Dir at DDA0_bit;


//variveis globais
bit f_ok,f_t1; //flags
int ciclo; //conta nmero de ciclos da converso
char mil,cent,dez,un,qcan;  //contadores de resultado decimais


//==============================================================================
//                ROTINAS DE INTERRUPO DO TIMER 1
//
//   Obs. Realiza a inteira converso e sinaliza no flag ->flag_ok=1
//   O resultado estar nos registradores mil,cent,dez e un.
//==============================================================================

 void Timer1_Interrupt() iv IVT_ADDR_TIMER1_OVF  {
      const int resolucao= 4096;  //resoluo de 12 bits (2^12)
      TCNT1H=0xfe ;   //Carga para tempo de interrupo a cada 55 useg
      TCNT1l=0x48;
      if(!f_ok) //se no foi lido sai da rotina
      if(!f_t1) {ciclo=0; mil='0';cent='0';dez='0';un='0'; f_t1=1;} //inicializao do conversor
      if(ACSR.ACO) PORTD.B5=1; else   {  //testa saida do comparador e altera o modulador de 1 bit pino 9
      PORTD.B5=0; 
      if(++un>'9'){un='0'; dez++;}  //incrementa contadores de resultado j em formato decimal
      if(dez>'9'){dez='0'; cent++;}
      if(cent>'9'){cent='0';mil++;}
      }//testa saida do comparador
      ciclo++;
      if(ciclo>=resolucao){f_ok=1;PORTD.B5=0;f_t1=0;}  //verifica se j acabou as 4096 converses
}



//==============================================================================
//                   ROTINAS PARA LCD
//==============================================================================
//clock para lcd
void lcd_clk(void){
 LCDEN=1;
 delay_us(1);//
 LCDEN=0;

 }

//envia nible para lcd
 void env_nib(char b){

 if(b&0x10)LCDD4=1; else LCDD4=0;
 if(b&0x20)LCDD5=1; else LCDD5=0;
 if(b&0x40)LCDD6=1; else LCDD6=0;
 if(b&0x80)LCDD7=1; else LCDD7=0;
 lcd_clk();
 }

 //envia comando para lcd
 void env_lcd_c(char val){
 LCDRS=0;
 env_nib(val);
 env_nib(val<<4);
 delay_ms(10);
}

//envia dados para lcd
 void env_lcd_d(char val){
 LCDRS=1;
 env_nib(val);
 env_nib(val<<4);
 delay_ms(1);
 }

void Lcd_String(char linha,char posicao, const char *frase){  //envia frases
  char a;
  if(linha==1)posicao+=0x7f; else posicao+=0xbf;
  env_lcd_c(posicao);
  for(a=0;frase[a]!=0;a++){
  env_lcd_d(frase[a]);
  }

}

//inicializao do lcd
void Init_Lcd(void){
 char a;

  LCDRS=0;
  LCDRS_Dir =1;
  LCDEN_Dir =1;
  LCDD4_Dir =1;
  LCDD5_Dir =1;
  LCDD6_Dir =1;
  LCDD7_Dir =1;
  LCDD4=0;//situao inicial resetados
  LCDD5=0;
  LCDD6=0;
  LCDD7=0;
  LCDEN=0;
  delay_ms(50);//delay inicial
  LCDD4=1; LCDD5=1;// setados
  for(a=0;a<3;a++) {lcd_clk(); delay_ms(5);}// envia 3,3,3
  LCDD4=0;//envia 2
  lcd_clk();//pulso em 'en'
  delay_ms(5);
  env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
  env_lcd_c(0x06);// no shift
  env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
  env_lcd_c(0x01);//limpa tela
  delay_ms(10);   //Delay

}
 /******************************************************************************
         ROTINA MAIN  (INICIALIZAES DE PORTS E REGISTRADORES)
 ******************************************************************************/

void main() {
  Init_Lcd();                        // Inicializa o  LCD
  env_lcd_c(1);  //limpa a tela
  Lcd_String(1,1,"Voltimetro_4_Can");   // Escreve nome do aparelho
  Lcd_String(2,1,"Larios.Tecnol.ws");   // Escreve nome do blog
  delay_ms(2000);// Tempo de apresentao da mensagem
  env_lcd_c(1);//limpa tela
  AIN0_DIR=0; //Entrada para comparador
  AIN1_DIR=0; //Entrada para comparador
  DIDR.AIN1D=1;//Desliga entrada digital para poupar energia
  DIDR.AIN0D=1;//Desliga entrada digital para poupar energia
  ACSR.ACBG=0; //Liga ain0 ao pino 12 (com 2 resistores formando divisor 1/2vcc)
 // ACSR.ACBG=1; //Liga ain0 ao vref
  ACSR.ACD=0; //Liga comparador analgico
  ACSR.ACIE=0;//Desliga interrupo do comparador
  SREG_I_bit = 1;  // Liga geral das interrupes
  TCCR1B = 1;    //divide 1  prescaller
  TOIE1_bit=1; //liga interrupo do timer 1 over flow
  DDD5_BIT=1;//Saida
  CANAL1_Dir=1;//Saida
  CANAL2_Dir=1;//Saida
  CANAL3_Dir=1;//Saida
  CANAL4_Dir=1;//Saida
  CANAL1=1; //primeira comverso no canal 1
  CANAL2=0;
  CANAL3=0;
  CANAL4=0;
  f_ok=0;
  qcan=0xff;
 // Lcd_String(1,1,"   Voltagem:    ");  //mensagem: "Voltagem:"
  /******************************************************************************
                     ROTINA DE REPETIO PRINCIPAL
 ******************************************************************************/
  for(;;){
    f_ok=0;
    if(++qcan>3)qcan=0;
    while(!f_ok);//aguarda converso da tenso
    if(qcan==0){ env_lcd_c(0x80);  env_lcd_d('A'); CANAL1=0;CANAL2=1; }
    if(qcan==1){ env_lcd_c(0x88);  env_lcd_d('B'); CANAL2=0;CANAL3=1; }
    if(qcan==2){ env_lcd_c(0xc0);  env_lcd_d('C'); CANAL3=0;CANAL4=1; }
    if(qcan==3){ env_lcd_c(0xc8);  env_lcd_d('D'); CANAL4=0;CANAL1=1; }
    env_lcd_d('=');
    if(mil=='0')env_lcd_d(' ');else env_lcd_d(mil);  //milhar ou elimina zero a esquerda
    env_lcd_d(cent); //centenas
    env_lcd_d('.');  //ponto
    env_lcd_d(dez); //dezenas
   // env_lcd_d(un);  //unidades
    env_lcd_d('V');  //sufixo 'V'

    delay_ms(70);   //delay entre converses
  }
}